import {useEffect, useState} from 'react';
import {MyIconInput, MyInputError} from './MyInput';
import {useSelector} from 'react-redux';
import {RootState} from '@redux/store';
import {
  accountNameOptions,
  passwordOptions,
  userNameOptions,
} from '@configs/types';
import reactStringReplace from 'react-string-replace';
import {MyIconButton} from './MyButton';
import {MyIcon} from './MyIcon';
import {Spinner} from '@gluestack-ui/themed';
import {apiBackendRequest, makeRequest} from '@helper/request';

export function usernameValid(username: string) {
  if (username.length < userNameOptions.minLength) {
    return false;
  } else if (!userNameOptions.isAllowed(username)) {
    return false;
  } else {
    return true;
  }
}

interface MyUsernameInputProps {
  username: string;
  setUsername: (username: string) => void;
}

export function MyUsernameInput(props: MyUsernameInputProps) {
  const lang = useSelector(
    (state: RootState) => state.appVariables.lang.commonInputs.username,
  );

  const [inputTouched, setInputTouched] = useState(false);

  const errorText = () => {
    if (props.username.length < userNameOptions.minLength) {
      return reactStringReplace(
        lang.errorLength,
        '${minLength}',
        (match, i) => {
          return userNameOptions.minLength.toString();
        },
      ).join('');
    } else if (!userNameOptions.isAllowed(props.username)) {
      return lang.errorUsernameInvalid;
    }
    return '';
  };

  return (
    <MyIconInput
      iconName="person"
      text={lang.label}
      value={props.username}
      onChangeText={text => {
        props.setUsername(text);
        setInputTouched(true);
      }}
      maxLength={userNameOptions.maxLength}
      helper={
        inputTouched &&
        !usernameValid(props.username) && <MyInputError text={errorText()} />
      }
    />
  );
}

export function passwordValid(password: string) {
  if (password.length < passwordOptions.minLength) {
    return false;
  } else if (!passwordOptions.isAllowed(password)) {
    return false;
  } else {
    return true;
  }
}

interface MyPasswordInputProps {
  password: string;
  setPassword: (password: string) => void;
  disableContainer?: boolean;
}

export function MyPasswordInput(props: MyPasswordInputProps) {
  const lang = useSelector(
    (state: RootState) => state.appVariables.lang.commonInputs.password,
  );

  const [inputTouched, setInputTouched] = useState(false);
  const [isPasswordVisible, setIsPasswordVisible] = useState(false);

  const togglePasswordVisibility = () =>
    setIsPasswordVisible(!isPasswordVisible);

  const errorText = () => {
    if (props.password.length < passwordOptions.minLength) {
      return reactStringReplace(lang.errorLength, '${minLength}', () => {
        return passwordOptions.minLength.toString();
      }).join('');
    } else if (!passwordOptions.isAllowed(props.password)) {
      return lang.errorPasswordInvalid;
    } else {
      return '';
    }
  };

  return (
    <MyIconInput
      text={lang.label}
      iconName="lock"
      secureTextEntry={!isPasswordVisible}
      value={props.password}
      maxLength={passwordOptions.maxLength}
      onChangeText={text => {
        props.setPassword(text);
        setInputTouched(true);
      }}
      rightComponent={
        <MyIconButton
          MyIconProps={{
            name: isPasswordVisible ? 'visibility-off' : 'visibility',
            size: 24,
          }}
          onPress={togglePasswordVisibility}
        />
      }
      helper={
        inputTouched &&
        !passwordValid(props.password) && <MyInputError text={errorText()} />
      }
      disableContainer={props.disableContainer}
    />
  );
}

export function accountNameValid(accountName: string) {
  if (accountName.length < accountNameOptions.minLength) {
    return false;
  } else if (!accountNameOptions.isAllowed(accountName)) {
    return false;
  } else {
    return true;
  }
}

export enum AccountNameAvailable {
  Loading,
  Available,
  NotAvailable,
}

interface MyAccountNameInputProps {
  accountName: string;
  setAccountName: (accountName: string) => void;
  checkAccountNameAvailability?: boolean;
  isAccountNameAvailable?: AccountNameAvailable;
  setIsAccountNameAvailable?: (
    isAccountNameAvailable: AccountNameAvailable,
  ) => void;
}

export function MyAccountNameInput(props: MyAccountNameInputProps) {
  const lang = useSelector(
    (state: RootState) => state.appVariables.lang.commonInputs.accountName,
  );
  const currentTheme = useSelector(
    (state: RootState) => state.nonSaveVariables.theme.colors,
  );

  const [inputTouched, setInputTouched] = useState(false);

  const rightComponent = () => {
    const closeIcon = (
      <MyIcon name="close" size={24} color={currentTheme.red600} />
    );

    if (!accountNameValid(props.accountName)) {
      return closeIcon;
    } else if (props.isAccountNameAvailable === AccountNameAvailable.Loading) {
      return <Spinner />;
    } else if (
      props.isAccountNameAvailable === AccountNameAvailable.Available
    ) {
      return <MyIcon name="check" size={24} color={currentTheme.green400} />;
    } else {
      return closeIcon;
    }
  };

  const errorText = () => {
    if (props.accountName.length < accountNameOptions.minLength) {
      return reactStringReplace(lang.errorLength, '${minLength}', () => {
        return accountNameOptions.minLength.toString();
      }).join('');
    } else if (!accountNameOptions.isAllowed(props.accountName)) {
      return lang.errorAccountNameInvalid;
    }
    return '';
  };

  if (props.checkAccountNameAvailability) {
    useEffect(() => {
      if (!accountNameValid(props.accountName)) return;

      const delay = 400;
      const timeoutId = setTimeout(() => {
        makeRequest({
          path: apiBackendRequest.CHECK_ACCOUNT_NAME,
          requestGET: {':accountName': props.accountName},
          response: {},
        })
          .then(() => {
            if (props.checkAccountNameAvailability) {
              if (props.setIsAccountNameAvailable) {
                props.setIsAccountNameAvailable(AccountNameAvailable.Available);
              }
            }
          })
          .catch(() => {
            if (props.setIsAccountNameAvailable) {
              props.setIsAccountNameAvailable(
                AccountNameAvailable.NotAvailable,
              );
            }
          });
      }, delay);

      // Cleanup the timeout on component unmount or when inputValue changes
      return () => clearTimeout(timeoutId);
    }, [props.accountName]);
  }

  return (
    <MyIconInput
      text={lang.label}
      iconName="person"
      value={props.accountName}
      onChangeText={text => {
        props.setAccountName(text);
        setInputTouched(true);

        if (props.setIsAccountNameAvailable) {
          props.setIsAccountNameAvailable(AccountNameAvailable.Loading);
        }
      }}
      maxLength={accountNameOptions.maxLength}
      rightComponent={props.checkAccountNameAvailability && rightComponent()}
      helper={
        inputTouched &&
        !accountNameValid(props.accountName) && (
          <MyInputError text={errorText()} />
        )
      }
    />
  );
}
