import {Box, Center, Spinner, Text, View} from '@gluestack-ui/themed';

import {EventID} from '@event/types';

import EventManager from '@event/EventManager';
import {MyScreenContainer} from '@components/MyScreenContainer';
import {SourceProp} from '@user/types';
import MyImage, {getImageURL} from '@components/MyImage';
import {Image, Linking} from 'react-native';
import {useEffect, useState} from 'react';
import {useSelector} from 'react-redux';
import {RootState} from '@redux/store';
import {MyButton} from '@components/MyButton';

function EventPageLoading() {
  // center also horizontally and vertically
  return (
    <View style={{flex: 1, justifyContent: 'center', alignItems: 'center'}}>
      <Spinner color="$secondary400" size={'large'} />
    </View>
  );
}

function EventPage({
  route,
  navigation,
}: {
  route: {params: {eventID: EventID}};
  navigation: any;
}) {
  const {eventID} = route.params;

  const event = EventManager.getEventSelector(eventID);

  const lang = useSelector((state: RootState) => state.appVariables.lang.event);

  const [isJoining, setIsJoining] = useState(false); // for join button (loading)

  useEffect(() => {
    navigation.setOptions({
      title: event.Name.data,
    });
  }, [event.Name.data]);

  if (event.Name.source === SourceProp.online) return <EventPageLoading />;

  const imageUrl = getImageURL([
    event.EventBannerPicture.hq,
    event.EventBannerPicture.lq,
  ]);

  const isJoinedToEvent = event.isJoined.data === 1;

  //const imageUrl =    'https://res.cloudinary.com/coin-nft/image/upload/c_limit,q_auto,w_329/f_auto/v1/cache/1/f2/65/f26573682335fe72b50f0f74041736905308345d6043abbac832856b3a4d2246-ZGMwNDMxZTktYTY4NS00OTQ1LWJiYmUtMDRhY2Q0YzUzMjAy?_a=ATCkFAA0';

  return (
    <MyScreenContainer scrollView={true}>
      {
        <>
          <Box
            marginBottom={20}
            style={{
              maxWidth: '100%',
              width: 444,
            }}>
            <MyImage
              source={{
                uri: imageUrl,
              }}
              style={{
                width: '100%',
                height: 250,
                borderRadius: 10,
              }}
              alt="Event banner"
            />

            <Text marginTop={10} fontWeight={'100'} color="$textLight400">
              {lang.info}
            </Text>
            <Box
              padding={10}
              backgroundColor="$backgroundDark300"
              borderRadius={10}>
              <Text>{event.Description.data}</Text>
            </Box>

            <Text marginTop={10} fontWeight={'100'} color="$textLight400">
              {lang.details}
            </Text>
            <Box
              padding={10}
              backgroundColor="$backgroundDark300"
              borderRadius={10}>
              <Text color="$textLight400">{event.UserLength.data}</Text>
              <Text color="$textLight400">{event.minAge.data}</Text>
              <Text color="$textLight400">{event.AddressText.data}</Text>
            </Box>

            <MyButton
              type="secondary"
              style={{marginTop: 20}}
              text={lang.tickets}
              onPress={() => {
                // open link in browser

                if (event.ButtonAction.data === undefined) return;
                {
                  const button = JSON.parse(event.ButtonAction.data);

                  if (button.url !== undefined) Linking.openURL(button.url);
                }
              }}
            />

            <MyButton
              type={isJoinedToEvent ? 'secondary' : 'primary'}
              style={{marginTop: 5}}
              isLoading={isJoining}
              disabled={isJoining}
              text={isJoinedToEvent ? lang.quit : lang.join}
              onPress={() => {
                setIsJoining(true);

                if (!isJoinedToEvent) {
                  EventManager.joinEvent(eventID)
                    .then(() => {
                      setIsJoining(false);
                    })
                    .catch(() => {
                      setIsJoining(false);
                    });
                } else {
                  EventManager.quitEvent(eventID)
                    .then(() => {
                      setIsJoining(false);
                    })
                    .catch(() => {
                      setIsJoining(false);
                    });
                }
              }}
            />

            <Text marginTop={10} fontWeight={'100'} color="$textLight400">
              debug
            </Text>
            <Box
              padding={10}
              backgroundColor="$backgroundDark300"
              borderRadius={10}>
              <Text color="$textLight400">{event.StartDate.data}</Text>
              <Text color="$textLight400">{event.EndDate.data}</Text>
            </Box>
          </Box>
        </>
      }
    </MyScreenContainer>
  );
}

export default EventPage;
